/* wptFileCBS.cpp
 *	Copyright (C) 2005 Timo Schulz
 *	Copyright (C) 2005 g10 Code GmbH
 *
 * This file is part of WinPT.
 *
 * WinPT is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2 
 * of the License, or (at your option) any later version.
 *  
 * WinPT is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with WinPT; if not, write to the Free Software Foundation, 
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <windows.h>
#include <stdio.h>
#include <malloc.h>
#include <errno.h>
#include <stdlib.h>
#include <sys/stat.h>

#include "gpgme.h"
#include "wptListView.h"
#include "wptGPG.h"
#include "wptFileManager.h"
#include "wptErrors.h"
#include "wptTypes.h"

void progress_callback (void *opaque, const char *what, int type, int off, int max);


/* Predefined read callback. */
static long
read_cb (void *handle, void *buffer, size_t size)
{
    file_data_t cb = (file_data_t)handle;
    struct progress_filter_s *pfx = (struct progress_filter_s *)cb->cb_value;
    int n = fread (buffer, 1, size, cb->handle);

    /* XXX: there is a sync problem with the progress dialog. */
    if (pfx)
	progress_callback (pfx, NULL, 0, cb->off, cb->size);
    cb->off += n;
    return n;
}


/* Predefined write callback. */
static long
write_cb (void *handle, const void *buffer, size_t size)
{
    file_data_t cb = (file_data_t)handle;
    int n = fwrite (buffer, 1, size, cb->handle);

    return n;
}


/* Create a new data -> file association with a static callback.
   @fname is the file which is associated to the object.
   @r_cb is the context which holds all information.
   @for_read is 1 if the file is opened for read only.
   Return value: 0 on success. */
gpgme_error_t
gpg_file_data_new (const char *fname, int for_read, file_data_t *r_cb)
		      
{
    gpgme_error_t err;
    file_data_t cb;
    FILE *f;

    f = fopen (fname, for_read?"rb" : "wb");
    if (!f)
	return gpgme_err_code_from_errno (errno);

    cb = (file_data_t)calloc (1, sizeof *cb);
    if (!cb)
	abort ();
    cb->cbs.read = read_cb;
    cb->cbs.write = write_cb;
    cb->handle = f;
    if (for_read) {
	struct stat st;
	if (fstat (fileno (f), &st))
	    BUG (NULL);
	cb->size = st.st_size;
	cb->off = 0;
    }

    err = gpgme_data_new_from_cbs  (&cb->dat, &cb->cbs, cb);
    if (err) {
	fclose (f);
	free (cb);
	return err;
    }

    *r_cb = cb;
    return err;
}


/* Activate the progress callback for the given object @ctx. */
void
gpg_file_data_set_cb (file_data_t ctx, struct progress_filter_s *pfx)
{
    ctx->cb_value = (void*)pfx;
}


/* Release the context in @cb. Close all internal handles if possible. */
void
gpg_file_data_release (file_data_t cb)
{
    if (!cb)
	return;
    if (cb->handle) {
	FILE *f = (FILE *)cb->handle;
	fclose (f);
    }
    if (cb->dat)
	gpgme_data_release (cb->dat);
    free (cb);
}
